/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Shell = imports.gi.Shell;
const St = imports.gi.St;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - Communications
 */
const StatusArea = Main.panel._statusArea || Main.panel.statusArea;

const DEFAULT_COMMUNICATION_APPLICATIONS = [
    'amsn.desktop',
    'emesene.desktop',
    'empathy.desktop',
    'gajim.desktop',
    'gwibber.desktop',
    'licq.desktop',
    'pidgin.desktop',
    'psi.desktop',
    'qutim.desktop',
    'skype.desktop',
    'xchat.desktop'
];

const KEY_SHOW_STATUSAREA = 'show-statusarea';

const STATUSAREA_ADVANCED = 1;

const PopupCommunicationApplicationsMenuSection = new Lang.Class({
    Name: 'PopupCommunicationApplicationsMenuSection',
    Extends: PopupMenu.PopupMenuSection,

    _init: function(wantIcon, wantOnlineAccounts) {
        this.parent();
        this._wantIcon = wantIcon || false;
        this._wantOnlineAccounts = wantOnlineAccounts || false;
        this._appSystem = Shell.AppSystem.get_default();
        this._connectAppSystem = this._appSystem.connect('installed-changed', Lang.bind(this, this._onInstalledChanged));
        this._onInstalledChanged();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
        }));
    },

    _onInstalledChanged: function() {
        this.removeAll();
        let apps = [];
        for each(let app in DEFAULT_COMMUNICATION_APPLICATIONS) {
            app = PanelTools.checkApplication(app);
            if (app) apps.push(app);
        }
        apps.sort(function(a, b) {
            return a.compare_by_name(b);
        });
        for each(let app in apps) {
            this.addMenuItem(new PanelTools.PopupApplicationMenuItem(app, this._wantIcon));
        }
        this.actor.visible = this.numMenuItems > 0;
        if (this._wantOnlineAccounts) {
            let app = PanelTools.checkApplication('gnome-online-accounts-panel.desktop');
            if (app) {
                this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
                this.addMenuItem(new PanelTools.PopupApplicationMenuItem(app));
            }
        }
    }
});

if (StatusArea.userMenu) const PopupIMStatusChooserMenuItem = new Lang.Class({
    Name: 'PopupIMStatusChooserMenuItem',
    Extends: imports.ui.userMenu.IMStatusChooserItem,

    _init: function(icon) {
        this.icon = icon || null;
        this.parent();
        this.connect('activate', Lang.bind(StatusArea.userMenu, StatusArea.userMenu._onMyAccountActivate));
        this._connectAccountMrg = StatusArea.userMenu._accountMgr.connect('most-available-presence-changed', Lang.bind(this, this._updatePresenceStatus));
        StatusArea.userMenu._accountMgr.prepare_async(null, Lang.bind(this, this._updatePresenceStatus))
        this.actor.connect('destroy', Lang.bind(this, function() {
            StatusArea.userMenu._accountMgr.disconnect(this._connectAccountMrg);
        }));
    },

    _updatePresenceStatus: function() {
        if (this.icon) this.icon.icon_name = StatusArea.userMenu._iconBox.get_child().get_icon_name();
    }
});

const PopupNotificationsSwitchMenuItem = new Lang.Class({
    Name: 'PopupNotificationsSwitchMenuItem',
    Extends: PopupMenu.PopupSwitchMenuItem,

    _init: function() {
        this.parent(_("Notifications"));
        this.connect('toggled', Lang.bind(this, function() {
            StatusArea.userMenu._notificationsSwitch.toggle();
        }));
        this._updateNotificationsSwitch();
        this._connectPresence = StatusArea.userMenu._presence.connectSignal('StatusChanged', Lang.bind(this, this._updateNotificationsSwitch));
        this.actor.connect('destroy', Lang.bind(this, function() {
            StatusArea.userMenu._presence.disconnectSignal(this._connectPresence);
        }));
    },

    _updateNotificationsSwitch: function() {
        this.setToggleState(StatusArea.userMenu._notificationsSwitch._switch.state);
    }
});

const CommunicationApplicationsSystemIndicator = new Lang.Class({
    Name: 'CommunicationApplicationsSystemIndicator',
    Extends: PanelMenu.SystemIndicator || PanelTools.SystemIndicator,

    _init: function() {
        this.parent();
        this._item = new PanelTools.PopupSubMenuMenuItem(_("Communication Applications"), true, { settings: false });
        this._item.menu.addMenuItem(new PopupCommunicationApplicationsMenuSection());
        this._item.setIconName('user-invisible-symbolic');
        this.menu.addMenuItem(this._item);
    },

    destroy: function() {
        this.menu.destroy();
        this.indicators.destroy();
    }
});

const CommunicationsIndicator = new Lang.Class({
    Name: 'CommunicationsIndicator',
    Extends: PanelTools.Button,

    _init: function() {
        this.parent(0.0, _("Communications"), { icon_name: 'user-invisible-symbolic' });
        if (StatusArea.userMenu) {
            this.menu.addMenuItem(new PopupIMStatusChooserMenuItem(this.icon));
            this.menu.addMenuItem(new PopupNotificationsSwitchMenuItem());
        } else {}
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this.menu.addMenuItem(new PopupCommunicationApplicationsMenuSection(true, true));
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onShowStatusArea));
        this._onShowStatusArea();
        this.actor.connect('destroy', Lang.bind(this, function() {
            this._appSystem.disconnect(this._connectAppSystem);
            this._settings.disconnect(this._connectSettings);
        }));
    },

    _onShowStatusArea: function() {
        this.actor.visible = this._settings.get_enum(KEY_SHOW_STATUSAREA) === STATUSAREA_ADVANCED;
    }
});

function getExtension() {
    return StatusArea.userMenu ? PanelTools.addToStatusArea('dschaerfCommunications', new CommunicationsIndicator(), PanelTools.getChildren('right') - 1, 'right') : null;
}
