/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*

    Copyright (C) 2012-2014  Dieter Schärf <ds@dschaerf.de>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.

 */

const Gettext = imports.gettext;
const Lang = imports.lang;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const UPowerGlib = imports.gi.UPowerGlib;

const ExtensionUtils = imports.misc.extensionUtils;

const Main = imports.ui.main;
const PopupMenu = imports.ui.popupMenu;

const Extension = ExtensionUtils.getCurrentExtension();

const Convenience = Extension.imports.convenience;
const PanelTools = Extension.imports.panelTools;
const SystemTools = Extension.imports.systemTools;

const _ = Gettext.domain(Convenience.CURRENT_GETTEXT_DOMAIN).gettext;

/*
    PanelManager - UserMenu
 */
const StatusArea = Main.panel._statusArea || Main.panel.statusArea;

const KEY_SHOW_STATUSAREA = 'show-statusarea';
const KEY_SHOW_STATUSMENU = 'show-statusmenu';

const STATUSAREA_DEFAULT = 0;
const STATUSAREA_ADVANCED = 1;
const STATUSAREA_ALTERNATE = 2;

const STATUSMENU_DEFAULT = 0;
const STATUSMENU_ICON = 1;

const SystemIndicator = new Lang.Class({
    Name: 'SystemIndicator',
    Extends: PanelTools.SystemIndicator,

    _init: function() {
        this.parent();
        this._indicator = this._addIndicator();
        this._indicator.icon_name = 'system-shutdown-symbolic';
        this._item = new PanelTools.PopupSubMenuMenuItem('', true, { settings: false, style_class: 'panelmanager-usermenu-avatar' });
        this._itemLockScreen = new PanelTools.PopupMenuItem(_("Lock Screen"));
        this._itemLockScreen.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateLockScreen));
        this._item.menu.addMenuItem(this._itemLockScreen);
        this._itemSwitchUser = new PanelTools.PopupMenuItem(_("Switch User"));
        this._itemSwitchUser.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateSwitchUser));
        this._item.menu.addMenuItem(this._itemSwitchUser);
        this._itemLogout = new PanelTools.PopupMenuItem(_("Log Out..."));
        this._itemLogout.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateLogout));
        this._item.menu.addMenuItem(this._itemLogout);
        this.menu.addMenuItem(this._item);
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        this._actions = new PanelTools.PopupActionMenuItem();
        this._actionPanelManagerSettings = this._actions.addAction('preferences-other-symbolic', 'Settings for PanelManager');
        this._actionPanelManagerSettings.connect('clicked', Lang.bind(StatusArea.userMenu, SystemTools.activatePanelManagerSettings));
        this._actionGNOMEShellRestart = this._actions.addAction('view-refresh-symbolic', _("GNOME Shell restart"));
        this._actionGNOMEShellRestart.connect('clicked', Lang.bind(StatusArea.userMenu, SystemTools.activateGNOMEShellRestart));
        this._actionTerminal = this._actions.addAction('utilities-terminal-symbolic', _("Terminal"));
        this._actionTerminal.connect('clicked', Lang.bind(StatusArea.userMenu, SystemTools.activateTerminal));
        this._actionSuspend = this._actions.addAction('media-playback-pause-symbolic', _("Suspend"));
        this._actionSuspend.connect('clicked', Lang.bind(StatusArea.userMenu, SystemTools.activateSuspend));
        this._actionHibernate = this._actions.addAction('media-playback-stop-symbolic', _("Hibernate"));
        this._actionHibernate.connect('clicked', Lang.bind(StatusArea.userMenu, SystemTools.activateHibernate));
        this._actionPowerOff = this._actions.addAction('system-shutdown-symbolic', _("Power Off..."));
        this._actionPowerOff.connect('clicked', Lang.bind(StatusArea.userMenu, SystemTools.activatePowerOff));
        this.menu.addMenuItem(this._actions);
        this._connectMenu = StatusArea.userMenu.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return;
            this._updateUserName();
            SystemTools.updateLockScreen.call(StatusArea.userMenu, this._itemLockScreen.actor);
            SystemTools.updateSwitchUser.call(StatusArea.userMenu, this._itemSwitchUser.actor);
            SystemTools.updateLogout.call(StatusArea.userMenu, this._itemLogout.actor);
            this._actionTerminal.visible = PanelTools.checkApplication('gnome-terminal.desktop');
            SystemTools.updateSuspend.call(StatusArea.userMenu, this._actionSuspend);
            SystemTools.updateHibernate.call(StatusArea.userMenu, this._actionHibernate);
            SystemTools.updatePowerOff.call(StatusArea.userMenu, this._actionPowerOff);
        }));
    },

    _updateUserName: function () {
        this._item.label.set_text(StatusArea.userMenu._user.is_loaded ? StatusArea.userMenu._user.get_real_name() : '');
        let iconFile = StatusArea.userMenu._user.get_icon_file();
        if (iconFile && GLib.file_test(iconFile, GLib.FileTest.EXISTS)) {
            let file = Gio.File.new_for_path(iconFile);
            let gicon = new Gio.FileIcon({ file: file });
            this._item.icon.gicon = gicon;
        } else {
            this._item.icon.icon_name = 'avatar-default-symbolic';
        }
    },

    destroy: function() {
        StatusArea.userMenu.menu.disconnect(this._connectMenu);
        this.menu.destroy();
        this.indicators.destroy();
    }
});

const UserMenu = new Lang.Class({
    Name: 'UserMenu',

    _init: function() {
        this._communicationApplications = new Extension.imports.communications.CommunicationApplicationsSystemIndicator();
        this._basicSettings = new Extension.imports.basicSettings.BasicSettingsSystemIndicator();
        this._system = new SystemIndicator();
        this._createIndicators();
        this._createSubMenu();
        this._settings = Convenience.getSettings(Convenience.CURRENT_SETTINGS_SCHEMA);
        this._connectSettings = [
            this._settings.connect('changed::'+KEY_SHOW_STATUSAREA, Lang.bind(this, this._onShowStatusArea)),
            this._settings.connect('changed::'+KEY_SHOW_STATUSMENU, Lang.bind(this, this._onShowStatusMenu))
        ];
        this._onShowStatusArea();
        this._connectMenu = StatusArea.userMenu.menu.connect('open-state-changed', Lang.bind(this, function(menu, open) {
            if (!open) return
            this._itemTerminal.actor.visible = PanelTools.checkApplication('gnome-terminal.desktop');
            SystemTools.updateSuspend.call(StatusArea.userMenu, this._itemSuspend.actor);
            SystemTools.updateHibernate.call(StatusArea.userMenu, this._itemHibernate.actor);
            SystemTools.updatePowerOff.call(StatusArea.userMenu, this._itemPowerOff.actor);
        }));
    },

    _createIndicators: function() {
        let box = StatusArea.userMenu.actor.get_children()[0];
        box.add(this._basicSettings.indicators, { y_align: St.Align.MIDDLE, y_fill: false });
        box.add(this._system.indicators, { y_align: St.Align.MIDDLE, y_fill: false });
    },

    _createSubMenu: function() {
        StatusArea.userMenu.menu.actor.add_style_class_name('panelmanager-usermenu');
        let menuItems = StatusArea.userMenu.menu._getMenuItems();
        for (let i = menuItems.length - 1; i >= 0; i--) {
            if (menuItems[i].label && menuItems[i].label.text === Gettext.gettext("Online Accounts")) {
                this._itemOnlineAccounts = menuItems[i];
            }
            if (menuItems[i] === StatusArea.userMenu._systemSettings || (menuItems[i].label && menuItems[i].label.text === Gettext.gettext("System Settings"))) {
                this._itemSettings = menuItems[i];
                StatusArea.userMenu.menu.addMenuItem(this._basicSettings.menu, i);
                StatusArea.userMenu.menu.addMenuItem(this._communicationApplications.menu, i);
            }
            if (menuItems[i] === StatusArea.userMenu._lockScreenItem) {
                StatusArea.userMenu.menu.box.remove_actor(menuItems[i].actor);
                this._itemLockScreen = new PopupMenu.PopupMenuSection();
                this._itemLockScreen.addMenuItem(menuItems[i]);
                StatusArea.userMenu.menu.addMenuItem(this._itemLockScreen, i);
            }
            if (menuItems[i] === StatusArea.userMenu._loginScreenItem) {
                StatusArea.userMenu.menu.box.remove_actor(menuItems[i].actor);
                this._itemSwitchUser = new PopupMenu.PopupMenuSection();
                this._itemSwitchUser.addMenuItem(menuItems[i]);
                StatusArea.userMenu.menu.addMenuItem(this._itemSwitchUser, i);
            }
            if (menuItems[i] === StatusArea.userMenu._logoutItem) {
                StatusArea.userMenu.menu.box.remove_actor(menuItems[i].actor);
                this._itemLogout = new PopupMenu.PopupMenuSection();
                this._itemLogout.addMenuItem(menuItems[i]);
                StatusArea.userMenu.menu.addMenuItem(this._itemLogout, i);
            }
            if (menuItems[i] === StatusArea.userMenu._suspendOrPowerOffItem) {
                StatusArea.userMenu.menu.box.remove_actor(menuItems[i].actor);
                this._itemSuspendOrPowerOff = new PopupMenu.PopupMenuSection();
                this._itemSuspendOrPowerOff.addMenuItem(menuItems[i]);
                StatusArea.userMenu.menu.addMenuItem(this._itemSuspendOrPowerOff, i);
                this._itemOthers = new PopupMenu.PopupMenuSection();
                this._itemGNOMEShellRestart = new PopupMenu.PopupMenuItem(_("GNOME Shell restart"));
                this._itemGNOMEShellRestart.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateGNOMEShellRestart));
                this._itemOthers.addMenuItem(this._itemGNOMEShellRestart);
                this._itemTerminal = new PopupMenu.PopupMenuItem(_("Terminal"));
                this._itemTerminal.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateTerminal));
                this._itemOthers.addMenuItem(this._itemTerminal, i);
                StatusArea.userMenu.menu.addMenuItem(this._itemOthers, i);
            }
            if (menuItems[i] === StatusArea.userMenu._installUpdatesItem) {
                StatusArea.userMenu.menu.box.remove_actor(menuItems[i].actor);
                this._itemInstallUpdates = new PopupMenu.PopupMenuSection();
                this._itemInstallUpdates.addMenuItem(menuItems[i]);
                StatusArea.userMenu.menu.addMenuItem(this._itemInstallUpdates, i);
            }
        }
        this._itemSeparator = new PopupMenu.PopupSeparatorMenuItem();
        StatusArea.userMenu.menu.addMenuItem(this._itemSeparator);
        this._itemPanelManagerSettings = new PopupMenu.PopupMenuItem(_("Settings for PanelManager"));
        this._itemPanelManagerSettings.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activatePanelManagerSettings));
        StatusArea.userMenu.menu.addMenuItem(this._itemPanelManagerSettings);
        this._itemSystem = new PopupMenu.PopupMenuSection();
        this._itemSuspend = new PopupMenu.PopupMenuItem(_("Suspend"));
        this._itemSuspend.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateSuspend));
        this._itemSystem.addMenuItem(this._itemSuspend);
        this._itemHibernate = new PopupMenu.PopupMenuItem(_("Hibernate"));
        this._itemHibernate.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activateHibernate));
        this._itemSystem.addMenuItem(this._itemHibernate);
        this._itemPowerOff = new PopupMenu.PopupMenuItem(_("Power Off..."), false, { style_class: 'popup-alternating-menu-item' });
        this._itemPowerOff.actor.add_style_pseudo_class('alternate');
        this._itemPowerOff.connect('activate', Lang.bind(StatusArea.userMenu, SystemTools.activatePowerOff));
        this._itemSystem.addMenuItem(this._itemPowerOff);
        StatusArea.userMenu.menu.addMenuItem(this._itemSystem);
        StatusArea.userMenu.menu.addMenuItem(this._system.menu);
    },

    _onShowStatusArea: function() {
        let showStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA);
        StatusArea.userMenu._statusChooser.actor.visible = showStatusArea !== STATUSAREA_ADVANCED;
        StatusArea.userMenu._notificationsSwitch.actor.visible = showStatusArea !== STATUSAREA_ADVANCED;
        if (showStatusArea === STATUSAREA_ALTERNATE) {
            if (!StatusArea.userMenu._notificationsSwitch._hbox) PanelTools.PopupSwitchMenuItemUpdate(StatusArea.userMenu._notificationsSwitch);
        } else {
            if (StatusArea.userMenu._notificationsSwitch._hbox) PanelTools.PopupSwitchMenuItemUpdate(StatusArea.userMenu._notificationsSwitch);
        }
        if (this._itemOnlineAccounts) this._itemOnlineAccounts.actor.visible = showStatusArea === STATUSAREA_DEFAULT;
        this._itemSettings.actor.visible = showStatusArea === STATUSAREA_DEFAULT;
        this._itemLockScreen.actor.visible = showStatusArea !== STATUSAREA_ALTERNATE && StatusArea.userMenu._lockScreenItem.actor.visible;
        this._itemSwitchUser.actor.visible = showStatusArea !== STATUSAREA_ALTERNATE && StatusArea.userMenu._loginScreenItem.actor.visible;
        this._itemLogout.actor.visible = showStatusArea !== STATUSAREA_ALTERNATE && StatusArea.userMenu._logoutItem.actor.visible;
        this._itemSuspendOrPowerOff.actor.visible = showStatusArea === STATUSAREA_DEFAULT && StatusArea.userMenu._suspendOrPowerOffItem.actor.visible;
        if (this._itemInstallUpdates) this._itemInstallUpdates.actor.visible = showStatusArea !== STATUSAREA_ALTERNATE && StatusArea.userMenu._installUpdatesItem.actor.visible;
        this._itemPanelManagerSettings.actor.visible = showStatusArea === STATUSAREA_DEFAULT;
        this._itemOthers.actor.visible = showStatusArea === STATUSAREA_ADVANCED;
        this._itemSystem.actor.visible = showStatusArea === STATUSAREA_ADVANCED;
        this._communicationApplications.menu.actor.visible = showStatusArea === STATUSAREA_ALTERNATE;
        this._basicSettings.menu.actor.visible = showStatusArea === STATUSAREA_ALTERNATE;
        this._system.menu.actor.visible = showStatusArea === STATUSAREA_ALTERNATE;
        this._onShowStatusMenu();
    },

    _onShowStatusMenu: function() {
        let showStatusArea = this._settings.get_enum(KEY_SHOW_STATUSAREA);
        let showStatusMenu = this._settings.get_enum(KEY_SHOW_STATUSMENU);
        StatusArea.userMenu._iconBox.visible = showStatusArea !== STATUSAREA_ADVANCED && showStatusMenu === STATUSMENU_DEFAULT;
        StatusArea.userMenu._name.visible = showStatusArea === STATUSAREA_DEFAULT && showStatusMenu === STATUSMENU_DEFAULT;
        this._basicSettings.indicators.visible = showStatusArea === STATUSAREA_ALTERNATE && showStatusMenu === STATUSMENU_DEFAULT;
        this._system.indicators.visible = showStatusArea !== STATUSAREA_DEFAULT || showStatusMenu === STATUSMENU_ICON;
        StatusArea.userMenu.actor.get_children()[0].set_style(showStatusArea === STATUSAREA_ALTERNATE ? 'spacing: 10px;' : 'spacing: 4px;');
    },

    _restoreIndicators: function() {
        StatusArea.userMenu.actor.get_children()[0].set_style('spacing: 4px;');
        StatusArea.userMenu._iconBox.show();
        StatusArea.userMenu._name.show();
        StatusArea.userMenu._updateUserName();
    },

    _restoreSubMenu: function() {
        StatusArea.userMenu.menu.actor.remove_style_class_name('panelmanager-usermenu');
        StatusArea.userMenu.menu.removeAll();
        StatusArea.userMenu._createSubMenu();
        StatusArea.userMenu._updateSwitch(StatusArea.userMenu._presence.status);
        StatusArea.userMenu._updateSwitchUser();
        StatusArea.userMenu._updateSuspendOrPowerOff();
        StatusArea.userMenu._updateLogout();
        StatusArea.userMenu._updateLockScreen();
        if (StatusArea.userMenu._updateInstallUpdates) StatusArea.userMenu._updateInstallUpdates();
    },

    destroy: function() {
        StatusArea.userMenu.menu.disconnect(this._connectMenu);
        for each(let connection in this._connectSettings) {
            this._settings.disconnect(connection);
        }
        this._system.destroy();
        this._basicSettings.destroy();
        this._communicationApplications.destroy();
        this._restoreSubMenu();
        this._restoreIndicators();
    }
});

function getExtension() {
    return StatusArea.userMenu ? new UserMenu() : null;
}
